/*
* Copyright (c) 2011-2012 by ps3dev.net
* This file is released under the GPLv2.
*/

#ifndef _TYPES_H_
#define _TYPES_H_

typedef char s8;
typedef unsigned char u8;
typedef short s16;
typedef unsigned short u16;
typedef int s32;
typedef unsigned int u32;
#ifdef _WIN32
typedef __int64 s64;
typedef unsigned __int64 u64;
#else
typedef long long int s64;
typedef unsigned long long int u64;
#endif

/*! Size of one sector. */
#define SECTOR_SIZE 0x200

#define BOOL int
#define TRUE 1
#define FALSE 0


//Endian swap for u16.
#define _ES16(val) \
	((u16)(((((u16)val) & 0xff00) >> 8) | \
	       ((((u16)val) & 0x00ff) << 8)))

//Endian swap for u32.
#define _ES32(val) \
	((u32)(((((u32)val) & 0xff000000) >> 24) | \
	       ((((u32)val) & 0x00ff0000) >> 8 ) | \
	       ((((u32)val) & 0x0000ff00) << 8 ) | \
	       ((((u32)val) & 0x000000ff) << 24)))

//Endian swap for u64.
#define _ES64(val) \
	((u64)(((((u64)val) & 0xff00000000000000ull) >> 56) | \
	       ((((u64)val) & 0x00ff000000000000ull) >> 40) | \
	       ((((u64)val) & 0x0000ff0000000000ull) >> 24) | \
	       ((((u64)val) & 0x000000ff00000000ull) >> 8 ) | \
	       ((((u64)val) & 0x00000000ff000000ull) << 8 ) | \
	       ((((u64)val) & 0x0000000000ff0000ull) << 24) | \
	       ((((u64)val) & 0x000000000000ff00ull) << 40) | \
	       ((((u64)val) & 0x00000000000000ffull) << 56)))


typedef struct {
	s32 dev;			        // devive handle
  u8 ps3_type;					//                        layer 1(ATA)       layer 2(ENCDEC)
                        // ps3 type: 1 = FAT_NAND(ATA: aes-cbc-192 / no VFLASH)
                        //           2 = FAT_NOR (ATA: aes-cbc-192 / VFLASH: aes-xts-128)
                        //           3 = SLIM_NOR(ATA: aes-xts-128 / VFLASH: aes-xts-128)
  u8 ata_k1[0x20];
  u8 ata_k2[0x20];
  u8 encdec_k1[0x20];
  u8 encdec_k2[0x20];
  u8 iv[0x10];
  s64 vflash_start;     // vflash
  s64 vflash_size;
  s64 hdd0_start;       // gameOS(UFS2)
  s64 hdd0_size;
  s64 hdd1_start;       // gameOS/swap(FAT32)
  s64 hdd1_size;
  s64 hdd1_free;
  s64 flash_start;      // vflash region 2(FAT16)
  s64 flash_size;
  s64 flash_free;
  s64 flash2_start;     // vflash region 3(FAT16)
  s64 flash2_size;
  s64 flash2_free;
  s64 flash3_start;     // vflash region 4(FAT12)
  s64 flash3_size;
  s64 flash3_free;
} ps3_context;


#endif  // _TYPES_H_
